;----------------------------------------------------------------------------

;G-200P PC commands processing module

;----------------------------------------------------------------------------

;Commands codes:

.equ	C_Nop		= 0x00		;No operation
.equ	C_Err		= 0x01		;Get Error code
.equ	C_Echo		= 0x02		;Get Echo
.equ	C_Info		= 0x03		;Get Device Information

.equ	C_SetCfg	= 0x04		;Set FPGA configuration mode
.equ	C_TxCfg		= 0x05		;Tx FPGA configuration data
.equ	C_TxDat		= 0x06		;Tx 32-bit data word to FPGA reg.
.equ	C_RxDat		= 0x07		;Rx 32-bit data word from FPGA reg.

;----------------------------------------------------------------------------

.CSEG	;Code segment

;----------------------------------------------------------------------------

;Process commands:

;----------------------------------------------------------------------------

mCmd:	cpi	CMD,C_Nop
	breq	Cmdr

	cpi	CMD,C_Err
	brne	Cmd1
	rcall	Do_Err
Cmd1:	cpi	CMD,C_Echo
	brne	Cmd2
	rcall	Do_Echo
Cmd2:	cpi	CMD,C_Info
	brne	Cmd3
	rcall	Do_Info

Cmd3:	cpi	CMD,C_SetCfg
	brne	Cmd4
	rcall	Do_SetCfg
Cmd4:	cpi	CMD,C_TxCfg
	brne	Cmd5
	rcall	Do_TxCfg
Cmd5:	cpi	CMD,C_TxDat
	brne	Cmd6
	rcall	Do_TxDat
Cmd6:	cpi	CMD,C_RxDat
	brne	Cmdr
	rcall	Do_RxDat
Cmdr:	ret

;----------------------------------------------------------------------------

;Command "C_Err" processing
;(transmit error code = Err_Tx):

;Tx_Dat[0] = Err_Tx - error code

Do_Err:	
	ldi	temp,Err_Tx		;load Err_Tx error code
	ldi	Cnt,1			;N
	sts	Tx_Str+x_Nbt,Cnt  	
	sts	Tx_Str+x_Cmd,CMD  	;command
	sts	Tx_Str+x_Dat,temp 	;error code
	rcall	UARTTX		  	;begin frame TX
	ldi	CMD,C_Nop	  	;end of command
	ret

;----------------------------------------------------------------------------

;Command "C_Echo" processing
;(return echo):

;Rx_Dat[0..Frame-1] - Received bytes

;Tx_Dat[0..Frame-1] - Echo bytes

Do_Echo:
	ldy	Rx_Str
	ldd	temp,Y+x_Nbt		;N
	push	temp
	tst	temp
	breq	ech_no

	mov	Cnt,temp
	ldy	Rx_Dat
	ldz	Tx_Dat
ech_c:	ld	temp,Y+
	st	Z+,temp
	dec	Cnt
	brne	ech_c

ech_no:	pop	Cnt			;N	
	sts	Tx_Str+x_Nbt,Cnt
	sts	Tx_Str+x_Cmd,CMD  	;command
	rcall	UARTTX		  	;begin frame TX
	ldi	CMD,C_Nop	  	;end of command
	ret

;----------------------------------------------------------------------------

;Command "C_Info" processing
;(transmit device info):

;Tx_Dat[0..up to 200] - Info string

Do_Info:
	table	Info
	ldy	Tx_Dat
	clr	Cnt
inf_c:	lpm	temp,Z+
	st	Y+,temp
	inc	Cnt
	tst	temp
	brne	inf_c			;Cnt = N

	sts	Tx_Str+x_Nbt,Cnt
	sts	Tx_Str+x_Cmd,CMD  	;command
	rcall	UARTTX		  	;begin frame TX
	ldi	CMD,C_Nop	  	;end of command
	ret
	
;----------------------------------------------------------------------------

;Command "C_SetCfg" processing
;(set configuration mode for FPGA):

;Tx_Dat[0] - Error code

Do_SetCfg:
	rcall	FPGA_SetCfg		;set FPGA configuration mode

	ldi	temp,Err_No		;error code
	ldi	Cnt,1			;N
	sts	Tx_Str+x_Nbt,Cnt
	sts	Tx_Str+x_Cmd,CMD  	;command
	sts	Tx_Str+x_Dat,temp 	;error code
	rcall	UARTTX		  	;begin frame TX
	ldi	CMD,C_Nop	  	;end of command
	ret

;----------------------------------------------------------------------------

;Command "C_TxCfg" processing
;(Tx FPGA configuration data):

;Rx_Dat[0..Frame-1] - data bytes

;Tx_Dat[0] - Error code

Do_TxCfg:
	ldy	Rx_Dat+0
	lds	Cnt,Rx_Str+x_Nbt
	rcall	FPGA_TxCfg		;Tx FPGA configuration data
	sts	Tx_Dat+1,temp		;status

	ldi	temp,Err_No		;error code
	ldi	Cnt,2			;N
	sts	Tx_Str+x_Nbt,Cnt
	sts	Tx_Str+x_Cmd,CMD  	;command
	sts	Tx_Str+x_Dat,temp 	;error code
	rcall	UARTTX		  	;begin frame TX
	ldi	CMD,C_Nop	  	;end of command
	ret

;----------------------------------------------------------------------------

;Command "C_TxDat" processing
;(Tx 32-bit data word to FPGA register):

;Rx_Dat[0] - address
;Rx_Dat[1] - data byte 1 (LSB)
;Rx_Dat[2] - data byte 2
;Rx_Dat[3] - data byte 3
;Rx_Dat[4] - data byte 4 (MSB)

;Tx_Dat[0] - Error code

Do_TxDat:
	ldy	Rx_Dat+0
	ld	temp,Y+			;address
	ld	tempL,Y+		;data
	ld	tempM,Y+
	ld	tempN,Y+
	ld	tempH,Y+
	rcall	FPGA_TxDat		;Tx data word to FPGA register

	ldy	Rx_Dat+0
	ld	temp,Y+			;address
	rcall	FPGA_RxDat		;Rx data word from FPGA register
	ld	temp,Y+
	cp	temp,tempL
	ld	temp,Y+
	cpc	temp,tempM
	ld	temp,Y+
	cpc	temp,tempN
	ld	temp,Y+
	cpc	temp,tempH
	breq	tx_ok
	ldi	temp,Err_Tx		;error code
	rjmp	tx_bd	

tx_ok:	ldi	temp,Err_No		;error code
tx_bd:	ldi	Cnt,1			;N
	sts	Tx_Str+x_Nbt,Cnt
	sts	Tx_Str+x_Cmd,CMD  	;command
	sts	Tx_Str+x_Dat,temp 	;error code
	rcall	UARTTX		  	;begin frame TX
	ldi	CMD,C_Nop	  	;end of command
	ret

;----------------------------------------------------------------------------

;Command "C_RxDat" processing
;(Rx 32-bit data word from FPGA register)

;Rx_Dat[0] - address

;Tx_Dat[0] - Error code
;Tx_Dat[1] - byte 1 (LSB)
;Tx_Dat[2] - byte 2
;Tx_Dat[3] - byte 3
;Tx_Dat[4] - byte 4 (MSB)

Do_RxDat:
	lds	temp,Rx_Dat+0		;address
	rcall	FPGA_RxDat		;Rx data word from FPGA register
	ldy	Tx_Dat+1
	st	Y+,tempL		;data
	st	Y+,tempM
	st	Y+,tempN
	st	Y+,tempH

	ldi	temp,Err_No		;error code
	ldi	Cnt,5			;N
	sts	Tx_Str+x_Nbt,Cnt
	sts	Tx_Str+x_Cmd,CMD  	;command
	sts	Tx_Str+x_Dat,temp 	;error code
	rcall	UARTTX		  	;begin frame TX
	ldi	CMD,C_Nop	  	;end of command
	ret

;----------------------------------------------------------------------------

Info:	.DB "G-200P V1.0", 0

;----------------------------------------------------------------------------

