;----------------------------------------------------------------------------

;G-200P FPGA control module

;----------------------------------------------------------------------------

.CSEG	;Code segment

;----------------------------------------------------------------------------

;FPGA subsystems init:

iFPGA:	ldi	temp,(1<<SPE) | (1<<DORD) | (1<<MSTR)
	out	SPCR,temp		;SPI enable, LSB first, master
	ldi	temp,(1<<SPI2X)	
	out	SPSR,temp		;double SPI speed

	ldi	temp,100
	rcall	mDel			;FPGA power-up delay 100 mS

	Port_NCLR_1			;release nCLR
	ret

;----------------------------------------------------------------------------

;Set configuration mode for FPGA:
  
FPGA_SetCfg:

	Port_nCONFIG_0			;nCONFIG <- 0
	ldi	Cnt,100
	rcall	uDel			;delay 100 uS
		
	Port_nCONFIG_1			;nCONFIG <- 1
	ldi	Cnt,100
	rcall	uDel			;delay 100 uS	

	ret

;----------------------------------------------------------------------------

;Tx FPGA configuration data byte:
;Input:	Y - pointer to configuration data
;	Cnt - number of bytes
;Out:	temp - status: 0 - OK, 1 - Done, 2 - Error 

FPGA_TxCfg:

	table	Encrypt
next_b:	ld	temp,Y+
	lpm	tempL,Z+
	eor	temp,tempL
	rcall	Spi_WR			;write configuration byte
	Skip_if_nSTAT_1
	rjmp	cfg_err
	Skip_if_CONFD_0
	rjmp	cfg_done
	dec	Cnt
	brne	next_b
	ldi	temp,0			;configuration in progress
	rjmp	cfg_r

cfg_done:
	ldi	Cnt,100
	rcall	uDel			;delay 100 uS	
	ldi	temp,1			;configuration done
	rjmp	cfg_r

cfg_err:
	ldi	temp,2			;configuration error
cfg_r:	ret

;----------------------------------------------------------------------------

;Tx 32-bit data word to FPGA register:
;Input:	temp - address
;	tempH:tempN:tempM:tempL - data word

FPGA_TxDat:

	Port_FS_0			;FS = 0
	rcall	Spi_WR			;write address
	mov	temp,tempL
	rcall	Spi_WR			;write byte 1 (LSB)
	mov	temp,tempM
	rcall	Spi_WR			;write byte 2
	mov	temp,tempN
	rcall	Spi_WR			;write byte 3
	mov	temp,tempH
	rcall	Spi_WR			;write byte 4 (MSB)
	Port_FS_1			;FS = 1
	ret

;----------------------------------------------------------------------------

;Rx 32-bit data word from FPGA register:
;Input:	temp - address
;Out:	tempH:tempN:tempM:tempL - data word

FPGA_RxDat:

	Port_FS_0			;FS = 0
	ori	temp,0x80		;read mode
	rcall	Spi_WR			;write address
	rcall	Spi_WR			;read byte 1 (LSB)
	mov	tempL,temp
	rcall	Spi_WR			;read byte 2
	mov	tempM,temp
	rcall	Spi_WR			;read byte 3
	mov	tempN,temp
	rcall	Spi_WR			;read byte 4 (MSB)
	mov	tempH,temp
	Port_FS_1			;FS = 1
	ret

;----------------------------------------------------------------------------

;Write/Read byte via SPI:
;Input:	temp - data byte to transmit
;Out:	temp - recieved data byte

Spi_WR:	out	SPDR,temp
spwait:	sbis	SPSR,SPIF
	rjmp	spwait
	in	temp,SPDR
	ret

;----------------------------------------------------------------------------

;Delay:
;Cnt - delay value, uS (2uS min)

uDel:	dec	Cnt
.if  FCLK > 8000000
	nop
	nop
.endif
.if  FCLK > 10000000
	nop
	nop
.endif
.if  FCLK > 12000000
	nop
	nop
.endif
.if  FCLK > 14000000
	nop
	nop
.endif
uDeo:	dec	Cnt
.if  FCLK > 4000000
	nop
	nop
.endif
.if  FCLK > 6000000
	nop
	nop
.endif
.if  FCLK > 8000000
	nop
	nop
.endif
.if  FCLK > 10000000
	nop
	nop
.endif
.if  FCLK > 12000000
	nop
	nop
.endif
.if  FCLK > 14000000
	nop
	nop
.endif
	nop
	brne	uDeo
	ret			

;----------------------------------------------------------------------------

;Delay:
;temp - delay value, mS

mDel:	ldi	tempH,5			;outer loop, 1mS
mDei:	ldi	Cnt,200			;inner loop, 200uS
	rcall	uDel
	dec	tempH
	brne	mDei
	wdr				;internal watchdog restart
	dec	temp
	brne	mDel
	ret

;----------------------------------------------------------------------------

Encrypt:

.include "Key.asm"	;FPGA configuration data encryption key

;----------------------------------------------------------------------------
