;----------------------------------------------------------------------------

;* Title	: G-200P programmable pulse generator
;* Version	: 1.00
;* Target	: ATmega8
;* Author	: wubblick@yahoo.com

;----------------------------------------------------------------------------

.include "m8def.inc"
.include "macros.mac"
.include "Header.asm"

;----------------------------------------------------------------------------

.CSEG	;Code segment

;----------------------------------------------------------------------------

;Interrupt Vectors:

.org	0
	rjmp	Init			;reset vector
.org	OC1Aaddr
	rjmp	Timer			;system timer vector
.org	SPMaddr+1			;skip vectors table

;----------------------------------------------------------------------------

;Program Execution Starts Here:

Init:	ldy	RAMEND	
	out	SPL,YL			;locate stack
	out	SPH,YH
	rcall	iWdog			;start internal watchdog
	rcall	iPorts			;ports init
	rcall	iVar			;variables init
	rcall	iTimer			;system timer init
	rcall	iFPGA			;FPGA subsystems init
	rcall	iWake			;Wake USB module init
	sei				;interrupts enable

;Main loop:

MAIN:	rcall	mCmd			;process PC commands
	rcall	mWdog			;watchdog restart
	rjmp 	MAIN			;loop

;----------------------------------------------------------------------------

;Internal watchdog init:

iWdog:	wdr
	ldi	temp,(1<<WDCE) | (1<<WDE)
	out	WDTCR,temp	
	ldi	temp,(1<<WDE) | (1<<WDP2)	
	out	WDTCR,temp		;watchdog enable, period 260 mS
	ret	

;----------------------------------------------------------------------------

;Watchdog restart:

mWdog:	bbrc	Flags,fUpd,nu
	clbr	Flags,fUpd
	wdr				;internal watchdog restart
nu:	ret

;----------------------------------------------------------------------------

;Ports init:
	
iPorts:	ldi	temp,PUPB
	out	PORTB,temp		;init PORTB and on/off pullup
	ldi	temp,DIRB	
	out	DDRB,temp		;set PORTB direction
	
	ldi	temp,PUPC
	out	PORTC,temp		;init PORTC and on/off pullup
	ldi	temp,DIRC
	out	DDRC,temp		;set PORTC direction

	ldi	temp,PUPD
	out	PORTD,temp		;init PORTD and on/off pullup
	ldi	temp,DIRD
	out	DDRD,temp		;set PORTD direction
	ret

;----------------------------------------------------------------------------

;Variables init:

iVar:	clr	Flags			;clear flags
	ret

;----------------------------------------------------------------------------

;System timer init:

iTimer:	clr	temp
	out	TCCR1A,temp		;OC disable, PWM disable
	out	GIMSK,temp
	ldi	temp,(1<<WGM12) | (1<<CS10)	
	out	TCCR1B,temp		;CTC, CK/1

	ldi	temp,byte2(T1DIV-1)
	out	OCR1AH,temp	
	ldi	temp,byte1(T1DIV-1)
	out	OCR1AL,temp

	in	temp,TIMSK
	ori	temp,(1<<OCIE1A)
	out	TIFR,temp		;Clear pending interrupts
	out	TIMSK,temp
	ret

;----------------------------------------------------------------------------

;System timer interrupt:

Timer:	push	temp
	in	temp,SREG		;save status register
	push	temp
	push	YL
	push	YH

	stbr	Flags,fUpd		;set update flag

	pop	YH
	pop	YL
	pop	temp
	out	SREG,temp		;restore status register
	pop	temp
	reti

;----------------------------------------------------------------------------

.include "Wake.asm"			;link Wake USB support module
.include "Commands.asm"			;link PC commands processing module
.include "Device.asm"			;link FPGA control module

;----------------------------------------------------------------------------
